/**
 * @fileoverview Shader attributes module.
 */
/**
 * The type of data that can be sent to a shader as an attribute.
 */
export declare type AttributeData = Float32List | Float32List[];
/**
 * Abstraction for sending data to shaders in attributes.
 */
export declare class Attribute<Data extends AttributeData> {
    protected readonly name_: string;
    protected readonly dimension_: number;
    protected data_: Data;
    protected buffer_: WebGLBuffer;
    constructor(name_: string, dimension_: number, data_: Data);
    /**
     * Send the attribute data to the shader.
     */
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
    /**
     * Get the number of vertices that will be sent to the shader.
     * This is only used if the shader is using gl.drawArrays(...).
     */
    length(): number;
}
/**
 * Type of function used to create attributes.
 */
declare type AttributeBuilder = (name: string, data: Float32List) => Attribute<Float32List>;
/**
 * Sends a float attribute to a shader.
 * @param name of the attribute in the shader
 * @param data the attribute will send
 */
export declare const FloatAttribute: AttributeBuilder;
/**
 * Sends a 2-dimensional vector attribute to a shader.
 * @param name of the attribute in the shader
 * @param data the attribute will send
 */
export declare const Vec2Attribute: AttributeBuilder;
/**
 * Sends a 3-dimensional vector attribute to a shader.
 * @param name of the attribute in the shader
 * @param data the attribute will send
 */
export declare const Vec3Attribute: AttributeBuilder;
/**
 * Sends a 4-dimensional vector attribute to a shader.
 * @param name of the attribute in the shader
 * @param data the attribute will send
 */
export declare const Vec4Attribute: AttributeBuilder;
/**
 * Abstraction for sending a matrix attribute to a shader.
 */
declare class MatrixAttribute extends Attribute<Float32List[]> {
    constructor(name: string, dimension: number, data: Float32List[]);
    /**
     * Send the data to the shader.
     */
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
}
/**
 * Type of function used to create matrix attributes.
 */
declare type MatAttributeBuilder = (name: string, data: Float32List[]) => MatrixAttribute;
/**
 * Sends a 2-dimensional matrix attribute to a shader.
 * @param name of the attribute in the shader
 * @param data the attribute will send
 */
export declare const Mat2Attribute: MatAttributeBuilder;
/**
 * Sends a 3-dimensional matrix attribute to a shader.
 * @param name of the attribute in the shader
 * @param data the attribute will send
 */
export declare const Mat3Attribute: MatAttributeBuilder;
/**
 * Sends a 4-dimensional matrix attribute to a shader.
 * @param name of the attribute in the shader
 * @param data the attribute will send
 */
export declare const Mat4Attribute: MatAttributeBuilder;
export {};
