/**
 * @fileoverview Module for GL context related operations.
 */
import { Cube } from './math';
/**
 * Get the current WebGL context.
 * If this is the first time you
 */
export declare const glContext: (canvas: HTMLCanvasElement) => WebGLRenderingContext;
/**
 * Create and compile a shader program.
 */
export declare const glProgram: (gl: WebGLRenderingContext, vertexSrc: string, fragmentSrc: string) => WebGLProgram;
/**
 * Send indices to the element array buffer.
 */
export declare const sendIndices: (gl: WebGLRenderingContext, indices: BufferSource) => void;
/**
 * Send an attribute to a shader's array buffer.
 */
export declare const sendAttribute: (gl: WebGLRenderingContext, program: WebGLProgram, buffer: WebGLBuffer, name: string, data: Float32Array, size: number) => void;
/**
 * Send a matrix attribute to a shader's array buffer.
 *
 * Matrix attributes are sent as vectors in column-major
 * order. Each element in "data" is the data for one column
 * for each vertex.
 */
export declare const sendMatrixAttribute: (gl: WebGLRenderingContext, program: WebGLProgram, buffer: WebGLBuffer, name: string, data: Float32Array[], dimension: number) => void;
/**
 * Different possible types of uniforms.
 */
export declare enum UniformType {
    BOOLEAN = "boolean",
    FLOAT = "float",
    INTEGER = "integer",
    VECTOR = "vector",
    MATRIX = "matrix",
    TEXTURE = "texture"
}
/**
 * Send a uniform to a shader that can be represented
 * with just a few bytes (boolean, float, integer).
 */
export declare const sendBytesUniform: (gl: WebGLRenderingContext, program: WebGLProgram, name: string, type: UniformType, data: number) => void;
/**
 * Sends a vector uniform to a shader.
 */
export declare const sendMatrixUniform: (gl: WebGLRenderingContext, program: WebGLProgram, name: string, dimension: number, data: Float32List) => void;
/**
 * Sends a vector uniform to a shader.
 */
export declare const sendVectorUniform: (gl: WebGLRenderingContext, program: WebGLProgram, name: string, dimension: number, data: Float32List) => void;
/**
 * Get the next available address for textures.
 */
export declare const newTextureOffset: (program: WebGLProgram, name: string) => number;
/**
 * Get if texture data is for a video source.
 */
export declare const isVideo: (data: TexImageSource | Cube<TexImageSource>) => boolean;
/**
 * Create a 2D texture with the provided data.
 */
export declare const texture2d: (gl: WebGLRenderingContext, data: TexImageSource) => WebGLTexture;
/**
 * Create a cube texture with the provided data.
 */
export declare const cubeTexure: (gl: WebGLRenderingContext, data: Cube<TexImageSource>) => WebGLTexture;
/**
 * Send a 2D texture as a uniform to a shader.
 */
export declare const send2DTexture: (gl: WebGLRenderingContext, program: WebGLProgram, name: string, offset: number, texture: WebGLTexture) => void;
/**
 * Send a cube texture as a uniform to a shader.
 */
export declare const sendCubeTexture: (gl: WebGLRenderingContext, program: WebGLProgram, name: string, offset: number, texture: WebGLTexture) => void;
/**
 * Creates a render buffer from a given frame buffer.
 */
export declare const renderBuffer: (gl: WebGLRenderingContext, fBuffer: WebGLFramebuffer, width: number, height: number) => WebGLRenderbuffer;
/**
 * Create a 2D texture that will sample from a framebuffer.
 */
export declare const texture2DFromFramebuffer: (gl: WebGLRenderingContext, frameBuffer: WebGLFramebuffer, width: number, height: number) => WebGLTexture;
/**
 * Generate a cube texture which will sample from a cube of framebuffers.
 */
export declare const cubeTextureFromFramebuffer: (gl: WebGLRenderingContext, frameBuffers: Cube<WebGLFramebuffer>, width: number, height: number) => WebGLTexture;
/**
 * Abstract type for the bounds of a viewport.
 */
export declare type Viewport = [number, number, number, number];
/**
 * Draws primitives to the bound buffers.
 */
export declare const glRender: (gl: WebGLRenderingContext, nVertices: number, clear: boolean, viewport: Viewport, mode: number, drawElements: boolean) => void;
