/**
 * @fileoverview Math module contains utility methods.
 *
 * All exported functions from this module should be pure functions.
 */
/**
 * Returns if a number is a power of two.
 */
export declare const isPowerOfTwo: (n: number) => boolean;
/**
 * A 3-dimensional vector type as a number array.
 */
export declare type Vector3 = [number, number, number];
/**
 * Checks if an object is a Vector3.
 */
export declare const isVector3: (obj: unknown) => obj is Vector3;
/**
 * A 4-dimensional vector type as a number array.
 */
export declare type Vector4 = [number, number, number, number];
declare type Vector = Vector3 | Vector4;
/**
 * Add two vectors.
 */
export declare const add: <V extends Vector>(a: V, b: V) => V;
declare type Matrix2 = [number, number, number, number];
/**
 * 3-dimensional matrix as a 9 element array.
 */
export declare type Matrix3 = [number, number, number, number, number, number, number, number, number];
/**
 * 4-dimensional matrix as a 16 element array.
 */
export declare type Matrix4 = [number, number, number, number, number, number, number, number, number, number, number, number, number, number, number, number];
/**
 * Matrix type.
 */
export declare type Matrix = Matrix2 | Matrix3 | Matrix4;
/**
 * Possible dimensions for vectors and matrices.
 */
export declare type Dimension = 2 | 3 | 4;
/**
 * Initialize an identity matrix with the provided dimension.
 */
export declare const identity: (dim: Dimension) => Matrix;
/**
 * Can be used for matrix-matrix and matrix-vector multiplication.
 */
export declare const multiply: <M extends Matrix, V extends Vector>(A: M, B: M | V) => M | V;
/**
 * Apply a 3D transmation to a 4-dimensional matrix, M.
 */
export declare const translate: (M: Matrix4, x: number, y: number, z: number) => Matrix4;
/**
 * Apply a scale transformation to a matrix.
 */
export declare const scale: <M extends Matrix>(A: M, ...args: number[]) => M;
/**
 * Apply a 3D rotation of theta radians around the given axis
 * to a 4D matrix.
 */
export declare const rotate: <M extends Matrix>(A: M, theta: number, axis_0: number, axis_1: number, axis_2: number) => M;
/**
 * Compute the 4D view matrix for a camera at
 * position "eye", looking at position "at", oriented
 * with "up" facing in the y+ direction.
 *
 * Based on https://github/toji/gl-matrix
 */
export declare const lookAt: (eye: Vector3, at: Vector3, up: Vector3, epsilon?: number) => Matrix4;
/**
 * Generate a 4D perspective matrix from
 * @param {fovy} vertical field of view (radians)
 * @param {aspect} width / height aspect ratio
 * @param {near} near bound of the frustum
 * @param {far} far bound of the frustum, can be Infinity or null
 */
export declare const perspective: (fovy: number, aspect: number, near: number, far: number | null) => Matrix4;
/**
 * Union type of all different faces for a cube.
 */
export declare type CubeFace = 'posx' | 'negx' | 'posy' | 'negy' | 'posz' | 'negz';
/**
 * Returns an array of all the cube face keys.
 */
export declare const cubeFaces: () => CubeFace[];
/**
 * Cube type has properties for each cube face.
 */
export declare type Cube<T> = Record<CubeFace, T>;
/**
 * Can either be the type, T, or a Cube of T.
 */
export declare type CubeOr<T> = T | Cube<T>;
/**
 * Test if an object is a Cube.
 */
export declare const isCube: <T>(obj: any) => obj is Record<CubeFace, T>;
/**
 * Build a cube by iterating over each face and calling
 * a given callback.
 */
export declare const cube: <T>(buildFace: (cf: CubeFace) => T) => Record<CubeFace, T>;
export {};
