/**
 * @fileoverview Shader uniforms module.
 */
import { UniformType, Viewport } from './gl';
import { Cube, CubeFace, CubeOr, Matrix3, Matrix4, Vector3, Vector4 } from './math';
/**
 * Either is type T or a nullary function that returns T.
 */
declare type IsOrReturns<T> = T | (() => T);
/**
 * Types of data that uniforms accept.
 */
export declare type UniformData = number | Float32List | CubeOr<TexImageSource>;
/**
 * Uniform base class. All uniforms are a superclass of this
 * class.
 */
export declare abstract class Uniform<Data extends UniformData> {
    protected readonly type_: UniformType;
    readonly name: string;
    protected dataOrCb_?: IsOrReturns<Data>;
    constructor(type_: UniformType, name: string, dataOrCb_?: IsOrReturns<Data>);
    /**
     * Check that the type of a uniform matches the expected type.
     */
    static checkType(u: Uniform<UniformData>, wantType: UniformType): void;
    /**
     * Get the data or callback to compute the data for the uniform.
     * Static because this method should only be used in phixl's internals.
     */
    static dataOrCallback<Data extends UniformData>(u: Uniform<Data>): IsOrReturns<Data>;
    /**
     * Set the data the uniform object will send to the shader.
     * @param dataOrCb new data or a callback to compute the new data.
     */
    set(dataOrCb: IsOrReturns<Data>): this;
    /**
     * Get the data the uniform will send to the shader.
     */
    data(): Data;
    /**
     * Send the uniform data to the shader.
     */
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
}
/**
 *
 */
declare class BytesUniform extends Uniform<number> {
    /**
     * Send the uniform data to the shader.
     */
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
}
/**
 * Send a boolean uniform to a shader.
 * @param name of the uniform in the shader
 * @param data the uniform will send to the shader
 */
export declare const BooleanUniform: (name: string, data?: IsOrReturns<number>) => BytesUniform;
/**
 * Send a float uniform to a shader.
 * @param name of the uniform in the shader
 * @param data the uniform will send to the shader
 */
export declare const FloatUniform: (name: string, data?: IsOrReturns<number>) => BytesUniform;
/**
 * Send a integer uniform to a shader.
 * @param name of the uniform in the shader
 * @param data the uniform will send to the shader
 */
export declare const IntegerUniform: (name: string, data?: IsOrReturns<number>) => BytesUniform;
/**
 * Type of uniform data for sequence uniforms, i.e. uniforms
 * which send a sequence of float values to a shader.
 */
declare type SequenceUniformType = UniformType.VECTOR | UniformType.MATRIX;
/**
 * Abstraction for sending sequence uniforms to a shader.
 */
declare class SequenceUniform extends Uniform<Float32List> {
    readonly dimension: number;
    constructor(type: SequenceUniformType, name: string, dimension: number, dataOrCb?: IsOrReturns<Float32List>);
    /**
     * Validate the sequence uniform's data.
     */
    private validateData_;
    /**
     * Send the data to a shader.
     */
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
    /**
     * Set the uniform's data.
     * @param dataOrCb new data or a callback which computes the data.
     */
    set(dataOrCb: IsOrReturns<Float32List>): this;
    /**
     * Check the dimension of a uniform.
     */
    static checkDimension(u: SequenceUniform, wantDimension: number): void;
}
/**
 * Type of function used to create sequence uniforms.
 */
declare type SequenceUniformBuilder = (name: string, data?: IsOrReturns<Float32List>) => SequenceUniform;
/**
 * Sends a 2-dimensional vector to a shader.
 * @param name of the uniform
 * @param data for the uniform
 */
export declare const Vec2Uniform: SequenceUniformBuilder;
/**
 * Sends a 3-dimensional vector to a shader.
 * @param name of the uniform
 * @param data for the uniform
 */
export declare const Vec3Uniform: SequenceUniformBuilder;
/**
 * Sends a 4-dimensional vector to a shader.
 * @param name of the uniform
 * @param data for the uniform
 */
export declare const Vec4Uniform: SequenceUniformBuilder;
/**
 * Sends a 2-dimensional matrix to a shader.
 * @param name of the uniform
 * @param data for the uniform
 */
export declare const Mat2Uniform: SequenceUniformBuilder;
/**
 * Sends a 3-dimensional matrix to a shader.
 * @param name of the uniform
 * @param data for the uniform
 */
export declare const Mat3Uniform: SequenceUniformBuilder;
/**
 * Sends a 4-dimensional matrix to a shader.
 * @param name of the uniform
 * @param data for the uniform
 */
export declare const Mat4Uniform: SequenceUniformBuilder;
/**
 * Type definition of the 2nd argument for ModelMatUniform.
 */
interface ModelMatOptions {
    scale?: number | Vector3;
    rotate?: Vector4;
    translate?: Vector3;
}
/**
 * Abstraction for sending a model matrix to a shader
 * as a uniform.
 */
declare class ModelMatUniformImpl extends SequenceUniform {
    private scaleMatrix_;
    private rotationMatrix_;
    private translation_;
    constructor(name: string, opts: ModelMatOptions);
    /**
     * Get the scale component for the model matrix as a 4D matrix.
     */
    scaleMatrix(): Matrix4;
    /**
     * Get the rotation component of the model matrix as a 3D matrix.
     */
    rotationMatrix(): Matrix3;
    /**
     * Gets the rotation matrix as a 4D matrix.
     */
    private rotationMatrix4_;
    /**
     * Gets the translation component of the model matrix as a 3D vector.
     */
    translation(): Vector3;
    /**
     * Get the model matrix this uniform will send as a 4D matrix.
     */
    private matrix_;
    /**
     * Apply a new scale to the model matrix.
     */
    scale(...args: number[]): void;
    /**
     * Set the scale of the model matrix.
     */
    setScale(...args: number[]): void;
    /**
     * Apply a rotation to the model matrix.
     * @param theta angle of rotation in radians
     * @param axis of rotation
     */
    rotate(theta: number, ...axis: Vector3): void;
    /**
     * Set the rotation component of the model matrix.
     * @param theta angle of rotation in radians
     * @param axis of rotation
     */
    setRotate(theta: number, ...axis: Vector3): void;
    /**
     * Apply a translation to the model matrix.
     */
    translate(...args: Vector3): void;
    /**
     * Set the translation component of the model matrix.
     */
    setTranslation(...args: Vector3): void;
}
/**
 * Sends a model matrix to a shader as a uniform that applies
 * a scale, rotation, and translation (in that order).
 */
export declare const ModelMatUniform: (name: string, opts?: ModelMatOptions) => ModelMatUniformImpl;
/**
 * Sends the resulting normal matrix for a given
 * model matrix to a shader.
 *
 * The normal matrix is the inverse transpose of
 * the rotation component of the model matrix.
 * Since for any rotation matrix R, it's inverse is
 * R^T.
 */
export declare const NormalMatUniform: (name: string, modelMat: ModelMatUniformImpl) => SequenceUniform;
/**
 * View matrix uniform with accessor and mutator methods for
 * the
 */
export declare class ViewMatUniformImpl extends SequenceUniform {
    private eye_;
    private at_;
    private up_;
    constructor(name: string, eye_: Vector3, at_: Vector3, up_: Vector3);
    /**
     * Checks if the provided data is a 3D vector.
     * @param data to validate
     */
    private static validateVector3;
    /**
     * Get the eye vector for the view matrix.
     */
    eye(): Vector3;
    /**
     * Get the look at vector for the view matrix.
     */
    at(): Vector3;
    /**
     * Get the up vector for the view matrix.
     */
    up(): Vector3;
    /**
     * Set the eye vector.
     */
    setEye(...eye: Vector3): void;
    /**
     * Set the look at vector.
     */
    setAt(...at: Vector3): void;
    /**
     * Set the up vector.
     */
    setUp(...up: Vector3): void;
}
/**
 * Sends a view matrix uniform for the given
 * eye, at, and up vectors.
 */
export declare const ViewMatUniform: (name: string, eye: Vector3, at: Vector3, up: Vector3) => ViewMatUniformImpl;
/**
 * Abstraction for a perspective matrix uniform.
 */
declare class PerspectiveMatUniformImpl extends SequenceUniform {
    private fovy_;
    private aspect_;
    private near_;
    private far_;
    constructor(name: string, fovy_: number, aspect_: number, near_: number, far_: number | null);
    /**
     * Get the field of view in radians.
     */
    fovy(): number;
    /**
     * Get the aspect ratio (width / height).
     */
    aspect(): number;
    /**
     * Get the near plane.
     */
    near(): number;
    /**
     * Get the far plane.
     */
    far(): number;
    /**
     * Set the field of view in radians.
     */
    setFovy(fovy: number): void;
    /**
     * Set the aspect ratio (width / height).
     */
    setAspect(aspect: number): void;
    /**
     * Set the near plane.
     */
    setNear(near: number): void;
    /**
     * Set the far plane.
     */
    setFar(far: number): void;
}
/**
 * Sends a perspective 4D matrix uniform.
 */
export declare const PerspectiveMatUniform: (name: string, fovy: number, aspect: number, near: number, far?: number | null) => PerspectiveMatUniformImpl;
/**
 * Possible types to use as texture data sources.
 */
export declare type TextureData = CubeOr<TexImageSource>;
/**
 * This type contains the WebGLFramebuffer and WebGLRenderbuffer used
 * to render a shader to a texture.
 */
interface Texture2DBuffers {
    frameBuffer: WebGLFramebuffer;
    renderBuffer: WebGLRenderbuffer;
}
/**
 * A cube of texture buffers for rendering a shader to a cube texture.
 */
declare type CubeTextureBuffers = {
    [k in keyof Texture2DBuffers]: Cube<Texture2DBuffers[k]>;
};
/**
 * Union type for buffers for 2D and cube textures.
 */
declare type TextureBuffers = Texture2DBuffers | CubeTextureBuffers;
/**
 * Abstraction for a texture uniform.
 */
export declare class TextureUniform<Data extends TextureData> extends Uniform<Data> {
    protected offset_: number;
    protected texture_: WebGLTexture;
    protected textureBuffers_: TextureBuffers;
    /**
     * Check if the uniform needs to build the texture.
     */
    protected shouldBuildTextureFromData_(): boolean;
    /**
     * Prepare the texture before using it in a shader.
     */
    prepare(gl: WebGLRenderingContext, p: WebGLProgram): void;
    /**
     * Get the WebGLFramebuffer and WebGLRenderbuffer to render a shader
     * to this texture.
     */
    buffers(gl: WebGLRenderingContext, v: Viewport): TextureBuffers;
    /**
     * Set new data for the uniform.
     * @param dataOrCb the new data or a callback computing the data.
     */
    set(dataOrCb: IsOrReturns<Data>): this;
}
/**
 * Tests if a uniform is for a texture.
 */
export declare const isTextureUniform: (u: Uniform<UniformData>) => u is TextureUniform<CubeOr<TexImageSource>>;
/**
 * Implementation of a 2D texture uniform.
 */
export declare class Texture2DUniformImpl extends TextureUniform<TexImageSource> {
    protected textureBuffers_: Texture2DBuffers;
    constructor(name: string, dataOrCb?: IsOrReturns<TexImageSource>);
    /**
     * Prepare the 2D texture before using it in a shader.
     */
    prepare(gl: WebGLRenderingContext, program: WebGLProgram): void;
    /**
     * Send the 2D texture to a shader.
     */
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
    /**
     * Get the buffers needed to render a shader to this texture.
     */
    buffers(gl: WebGLRenderingContext, viewport: Viewport): Texture2DBuffers;
}
/**
 * Sends a 2D texture uniform to a shader.
 */
export declare const Texture2DUniform: (name: string, data?: IsOrReturns<TexImageSource>) => Texture2DUniformImpl;
/**
 * Implementation of a cube texture.
 */
export declare class CubeTextureImpl extends TextureUniform<Cube<TexImageSource>> {
    protected textureBuffers_: CubeTextureBuffers;
    /**
     * Validate that the data is a cube.
     */
    private validateData;
    /**
     * Set the data of the cube texture.
     * @param dataOrCb new data or callback that computes the data.
     */
    set(dataOrCb: IsOrReturns<Cube<TexImageSource>>): this;
    /**
     * Prepare the cube texture to be used in a shader.
     */
    prepare(gl: WebGLRenderingContext, program: WebGLProgram): void;
    /**
     * Send the cube texture to a shader.
     */
    send(gl: WebGLRenderingContext, program: WebGLProgram): void;
    /**
     * Get a Cube of buffers needed to render a shader to the cube texture.
     */
    buffers(gl: WebGLRenderingContext, viewport: Viewport): CubeTextureBuffers;
}
/**
 * Sends a cube texture uniform to a shader.
 */
export declare const CubeTextureUniform: (name: string, data?: IsOrReturns<Cube<TexImageSource>>) => CubeTextureImpl;
/**
 * Tests if a uniform is a cube texture uniform.
 */
export declare const isCubeTextureUniform: (u: TextureUniform<CubeOr<TexImageSource>>) => u is CubeTextureImpl;
/**
 * Implementation of a cube camera uniform.
 */
export declare class CubeCameraUniformImpl extends CubeTextureImpl {
    private position_;
    private viewMat_;
    private perspectiveMat_;
    private static upVectors;
    private static atVectors;
    constructor(name: string, position_: Vector3, viewMat_: ViewMatUniformImpl, perspectiveMat_: PerspectiveMatUniformImpl);
    /**
     * You should not use this method with a cube camera uniform.
     */
    set(dataOrCb: IsOrReturns<Cube<TexImageSource>>): never;
    /**
     * Render a scene to a cube camera texture. Should only be used by
     * phixl internals.
     */
    render(renderShader: (cf: CubeFace) => void): void;
}
/**
 * Sends a cube texture to a shader rendered from
 * a cube camera.
 */
export declare const CubeCameraUniform: (name: string, position?: Vector3, viewMat?: ViewMatUniformImpl, perspectiveMat?: PerspectiveMatUniformImpl) => CubeCameraUniformImpl;
export {};
